/************************************************************************/
/* TaxSolve_NJ_1040_2007.c - 						*/
/* Copyright (C) 2007 - Aston Roberts					*/
/* 									*/
/* Compile:   gcc taxsolve_NJ_1040_2006.c -o taxsolve_NJ_1040_2006	*/
/* Run:	      ./taxsolve_NJ_1040_2006  NJ_1040_2006.dat 		*/
/* 									*/
/* GNU Public License - GPL:						*/
/* This program is free software; you can redistribute it and/or	*/
/* modify it under the terms of the GNU General Public License as	*/
/* published by the Free Software Foundation; either version 2 of the	*/
/* License, or (at your option) any later version.			*/
/* 									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU	*/
/* General Public License for more details.				*/
/* 									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA		*/
/* 02111-1307 USA							*/
/* 									*/
/* Aston Roberts 1-2-2004	aston_roberts@yahoo.com			*/
/* Updates by:  Aston Roberts 1-18-2005					*/
/*    2-14-05   BWB							*/
/*	2-12-07	Aston Roberts						*/
/*      2-24-06 Further updates BWB					*/
/*	2-1-08	Aston Roberts						*/
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include "taxsolve_routines.c"

float thisversion=5.1;
double A[MAX_LINES], S[MAX_LINES], E[MAX_LINES];

#define SINGLE 		        1
#define MARRIED_FILLING_JOINTLY 2
#define MARRIED_FILLING_SEPARAT 3
#define HEAD_OF_HOUSEHOLD       4
#define WIDOW		        5

double TaxRateFunction( double x, int status )
{

 if ((status==SINGLE) || (status==MARRIED_FILLING_SEPARAT))	/* Single, Married/sep */
  {
   if (x<20000.0)  return x * 0.014; else
   if (x<35000.0)  return x * 0.0175  -   70.0; else
   if (x<40000.0)  return x * 0.035   -  682.5; else
   if (x<75000.0)  return x * 0.05525 - 1492.5; else
   if (x<500000.0) return x * 0.0637  - 2126.25;
   else		   return x * 0.0897 - 15126.25;
  }
 else
 if ((status==MARRIED_FILLING_JOINTLY) || (status==HEAD_OF_HOUSEHOLD) || (status==WIDOW))	 /* Married/Joint, HouseHead, widower. */
  {
   if (x<20000.0)  return x * 0.014; else
   if (x<50000.0)  return x * 0.0175   -  70.0; else
   if (x<70000.0)  return x * 0.0245   - 420.0; else
   if (x<80000.0)  return x * 0.035   - 1154.5; else
   if (x<150000.0) return x * 0.05525 - 2775.0; else
   if (x<500000.0) return x * 0.0637  - 4042.5;
   else		   return x * 0.0897 - 17042.5;
  }
 else {printf("Status not covered.\n"); exit(0);}
}




/*----------------------------------------------------------------------------*/

int main( int argc, char *argv[] )
{
 int i, j, k;
 char word[1000], outfname[1000];
 int status=0;
 time_t now;
 int L12a=0, L12b=0;
 double L27a=0.0, L27b=0.0, L29a=0.0, L29b=0.0, L36a=0.0;
 double Ab[10], proptxcredit;
 double F[10], Fb[10];	/* Schedule F, added by BWB. */

 /* Intercept any command-line arguments. */
 printf("NJ 1040 2007 - v%3.1f\n", thisversion);
 i = 1;  k=1;
 while (i < argc)
 {
  if (strcmp(argv[i],"-verbose")==0)  verbose = 1;
  else
  if (k==1)
   {
    infile = fopen(argv[i],"r");
    if (infile==0) {printf("ERROR: Parameter file '%s' could not be opened.\n", argv[i]); exit(1);}
    k = 2;
    /* Base name of output file on input file. */
    strcpy(outfname,argv[i]);
    j = strlen(outfname)-1;
    while ((j>=0) && (outfname[j]!='.')) j--;
    if (j<0) strcat(outfname,".out"); else strcpy(&(outfname[j]),".out");
    outfile = fopen(outfname,"w");
    if (outfile==0) {printf("ERROR: Output file '%s' could not be opened.\n", outfname); exit(1);}
    printf("Writing results to file:  %s\n", outfname);
   }
  else {printf("Unknown command-line parameter '%s'\n", argv[i]); exit(1);}
  i = i + 1;
 }

 if (infile==0) {printf("Error: No input file on command line.\n"); exit(1);}

 /* Pre-initialize all lines to zeros. */
 for (i=0; i<MAX_LINES; i++) 
  {
   L[i] = 0.0;
   A[i] = 0.0;
   S[i] = 0.0;
   E[i] = 0.0;
  }

 /* Accept parameters from input file. */
 /* Expect  NJ-1040 lines, something like:
	Title:  NJ 1040 1999 Return
	L14		{Wages}
	L15a		{Interest}
	L16		{Dividends}
	L18		{Capital Gains}
	S1		{Property Tax}
	L42		{Witheld tax, from W-2}
*/


 /* Accept Form's "Title" line, and put out with date-stamp for records. */
 read_line( infile, word );
 now = time(0);
 fprintf(outfile,"\n%s		%s\n", word, ctime( &now ));

 /* get_parameter(infile, kind, x, mesage ) */
 get_parameter( infile, 's', word, "Status" );
 get_parameter( infile, 'l', word, "Status ?");
 if (strncasecmp(word,"Single",4)==0) status = SINGLE; else
 if (strncasecmp(word,"Married/Joint",13)==0) status = MARRIED_FILLING_JOINTLY; else
 if (strncasecmp(word,"Married/Sep",11)==0) status = MARRIED_FILLING_SEPARAT; else
 if (strncasecmp(word,"Head_of_House",4)==0) status = HEAD_OF_HOUSEHOLD; else
 if (strncasecmp(word,"Widow",4)==0) status = WIDOW;
 else
  { 
   printf("Error: unrecognized status '%s'. Must be: Single, Married/joint, Married/sep, Head_of_house, Widow(er)\nExiting.\n", word); 
   exit(0); 
  }
 switch (status)
 {
  case SINGLE: 			fprintf(outfile,"Status = Single (%d)\n", status); break;
  case MARRIED_FILLING_JOINTLY: fprintf(outfile,"Status = Married/Joint (%d)\n", status); break;
  case MARRIED_FILLING_SEPARAT: fprintf(outfile,"Status = Married/Sep (%d)\n", status); break;
  case HEAD_OF_HOUSEHOLD: 	fprintf(outfile,"Status = Head_of_Household (%d)\n", status); break;
  case WIDOW: 		  	fprintf(outfile,"Status = Widow(er) (%d)\n", status); break;
 }

 get_parameter( infile, 's', word, "L6" );	/* Exemptions, self/spouse. */
 get_parameter( infile, 'i', &j, "L6"); 
 L[6] = j;
 shownum(6); 

 get_parameter( infile, 's', word, "L7" );	/* Exemptions, Over 65. */
 get_parameter( infile, 'i', &j, "L7"); 
 L[7] = j;
 shownum(7); 

 get_parameter( infile, 's', word, "L8" );	/* Exemptions, Blind/disabled. */
 get_parameter( infile, 'i', &j, "L8"); 
 L[8] = j;
 shownum(8); 

 get_parameter( infile, 's', word, "L9" );	/* Exemptions, children. */
 get_parameter( infile, 'i', &j, "L9"); 
 L[9] = j;
 shownum(9); 

 get_parameter( infile, 's', word, "L10" );	/* Exemptions, other dependents. */
 get_parameter( infile, 'i', &j, "L10"); 
 L[10] = j;
 shownum(10); 

 get_parameter( infile, 's', word, "L11" );	/* Exemptions, college kids. */
 get_parameter( infile, 'i', &j, "L11"); 
 L[11] = j;
 shownum(11); 

 L12a = L[6] + L[7] + L[8] + L[11];
 fprintf(outfile,"L12a = %d\n", L12a);
 L12b = L[9] + L[10];
 fprintf(outfile,"L12b = %d\n", L12b);

 GetLineF( "L14", &L[14] );	/* Wages. */

 GetLineF( "L15a", &L[15] );	/* Taxable Interest. */

 /* Form asks for tax-exempt income, but does not use it. */

 GetLineF( "L16", &L[16] );	/* Dividends. */

 GetLineF( "L17", &L[17] );	/* Business profits, Fed Sched C. */

 GetLine( "L18", &L[18] );	/* Capital Gains . */
 if (L[18] < 0.0) L[18] = 0.0;
 showline(18);

 GetLineF( "L19", &L[19] );	/* Pensions, Annuities, and IRA Withdrawals (pg 23). */

 GetLineF( "L20", &L[20] );	/* Partnership income. (See pg 26.) */

 GetLineF( "L21", &L[21] );	/* S Corporation income. (See pg 26.) */

 GetLineF( "L22", &L[22] );	/* Rent, royalty, patents income. (Sched C line 3.) */

 GetLineF( "L23", &L[23] );	/* Net gambling winnings. */

 GetLineF( "L24", &L[24] );	/* Alimony and maintenance payments RECEIVED. */

 GetLineF( "L25", &L[25] );	/* Other (See pg 27). */

 L[26] = L[14] + L[15] + L[16] + L[17] + L[18] + L[19] + L[20] + L[21] + L[22] + L[23] + L[24] + L[25];
 showline_wmsg(26,"Total Income");	/* Total Income. */

 GetLineF( "L27a", &L27a );	/* Pension Exclusion (See pg 27). */
 GetLineF( "L27b", &L27b );	/* Other Retirement Income Exclusion (See worksheet pg 28). */
 L[27] = L27a + L27b;
 showline(27);

 L[28] = L[26] - L[27];
 showline_wmsg(28,"NJ Gross Income");

 L29a = L12a * 1000.0;
 fprintf(outfile," L29a = %6.2f\n", L29a );
 L29b = L12b * 1500.0;
 fprintf(outfile," L29b = %6.2f\n", L29b );
 L[29] = L29a + L29b;
 fprintf(outfile,"L29c = %6.2f	Total Exemption Amount\n", L[29] );

 fprintf(outfile,"\n");
 GetLine( "E1", &E[1] );	/* Medical Expenses (See pg 29). */
 showline_wrksht('E',1,E);
 E[2] = 0.02 * L[28];
 showline_wrksht('E',2,E);
 E[3] = NotLessThanZero( E[1] - E[2] );
 showline_wrksht('E',3,E);
 GetLine( "E4", &E[4] );      /* Qualified Archer MSA contributions from Federal Form 8853 */
 showline_wrksht('E',4,E);
 GetLine( "E5", &E[5] );      /* Amount of self-employed health insurance deduction */
 showline_wrksht('E',5,E);
 E[6] = NotLessThanZero( E[3] + E[4] + E[5] );
 showline_wrksht('E',6,E);
 fprintf(outfile,"\n");
 L[30] = E[6];
 showline_wmsg(30," Medical Expenses Worksheet E (See pg 29)");
 /* end of Worksheet E */

 GetLineF( "L31", &L[31] );	/* Alimony and maintenance payments PAYED. */

 GetLineF( "L32", &L[32] );	/* Qualified Conservation Contribution. */

 GetLineF( "L33", &L[33] );	/* Health Enterprise Zone Deduction. */

 L[34] = L[29] + L[30] + L[31] + L[32] + L[33];
 showline_wmsg(34,"Total Exemptions and Deductions");

 /* Taxable income. */
 L[35] = L[28] - L[34];
 if (L[35] > 0.0)
  showline_wmsg(35,"(Taxable Income)");

 GetLineF( "L36a", &L36a );	/* Property Tax Paid. */
 L[36] = L36a;

 fprintf(outfile,"\n");  /* Tax deduction worksheet. */
 S[1] = L[36];		 /* Schedule-1 Worksheet for Property Tax. */
 showline_wrksht('S',1,S);

 S[2] = smallerof( S[1], 10000.0 );
 showline_wrksht('S', 2, S);

 S[3] = L[35];
 showline_wrksht('S',3,S);
 S[4] = S[2];
 showline_wrksht('S',4,S);
 S[5] = S[3] - S[4];
 showline_wrksht('S',5,S);
 S[6] = TaxRateFunction( S[5], status );
	fprintf(outfile,"		6a. Tax Amount with PropTax Deduction=%6.2f\n", S[6]);
	fprintf(outfile,"		6b. Tax Amount without PTax Deduction=%6.2f\n", TaxRateFunction( S[3], status ));
 showline_wrksht('S',6,S);
 S[7] = TaxRateFunction( S[3], status ) - S[6];
 showline_wrksht('S',7,S);

 if (status != MARRIED_FILLING_SEPARAT)
  proptxcredit = 50.0;
 else
  proptxcredit = 25.0;

 if (S[7] >= proptxcredit) 
  {
   fprintf(outfile," Yes: Take PropTax Deduction\n\n");
   L[36] = S[4];
   // fprintf(outfile,"L36c = %6.2f\n", L[36]);
   L[37] = S[5];
   L[38] = S[6];
   L[48] = 0.0;
   showline(36);
  }
 else
  {
   fprintf(outfile," No: Take PropTax Credit\n\n");
   L[36] = 0.0;
   // fprintf(outfile,"L36c = %6.2f\n", L[36]);
   L[37] = S[3];
   L[38] = TaxRateFunction( S[3], status );
   L[48] = proptxcredit;
   showline(36);
  }
 
 GetLine( "A1", &A[1] );	/* Income taxed by other jurisdictions, if any. */
 GetLine( "A9a", &A[9] );	/* Tax paid to other jurisdictions on that income, if any. */
 if (A[1] > 0.0)
  { /*Sched-A*/
   fprintf(outfile,"\nSchedule A:\n");
   showline_wrksht('A',1,A); 
   A[2] = L[28];
   showline_wrksht('A',2,A); 
   A[3] = smallerof( 1.0, (A[1] / A[2]) );
   fprintf(outfile," A3 = %6.2f %%\n", 100.0 * A[3]);
   A[4] = L[35];
   fprintf(outfile," A4a = %6.2f	A4b = %6.2f\n", A[4], A[4]);
   A[5] = L36a;
   fprintf(outfile," 5a = %6.2f\n", A[5] );
   A[5] = smallerof( A[5], 10000.0 );
   fprintf(outfile," A5a = %6.2f	A5b = %6.2f\n", A[5], 0.0);
   A[6] = A[4] - A[5];
   Ab[6] = A[4] - 0.0;
   fprintf(outfile," A6a = %6.2f	A6b = %6.2f\n", A[6], Ab[6]);
   A[7] = TaxRateFunction( A[6], status );
   Ab[7] = TaxRateFunction( Ab[6], status );
   fprintf(outfile," A7a = %6.2f	A7b = %6.2f\n", A[7], Ab[7] );
   A[8] = A[3] * A[7];
   Ab[8] = A[3] * Ab[7];
   fprintf(outfile," A8a = %6.2f	A8b = %6.2f\n", A[8], Ab[8] );
   fprintf(outfile,"  9a = %6.2f\n", A[9]);
   Ab[9] = smallerof( smallerof( A[9], Ab[8] ), L[38] );
   A[9] = smallerof( smallerof( A[9], A[8] ), L[38] );
   fprintf(outfile," A9a = %6.2f	A9b = %6.2f\n", A[9], Ab[9] );

   if ((L[36] == 0.0) && (L[48] == 0.0))	/* Eligible for prop-tax ded. or credit? */
    L[39] = A[9];				/*  No. */
   else 					/* Otherwise do Sched-F. */
    { /* Schedule F - Added by BWB. */
      F[1] = A[7];
      Fb[1] = Ab[7];
      fprintf(outfile," F1a = %6.2f	F1b = %6.2f\n", F[1], Fb[1]);
      F[2] = A[9];
      Fb[2] = Ab[9];
      fprintf(outfile," F2a = %6.2f	F2b = %6.2f\n", F[2], Fb[2]);
      F[3] = F[1] - F[2];
      Fb[3] = Fb[1] - Fb[2];
      fprintf(outfile," F3a = %6.2f	F3b = %6.2f\n", F[3], Fb[3]);
      Fb[4] = Fb[3] - F[3];
      fprintf(outfile,"  F4 = %6.2f\n", Fb[4]);
      if (Fb[4] >= proptxcredit)
       {
        fprintf(outfile," Sched-F, Yes: Take PropTax Deduction\n\n");
        L[36] = A[5];
	// fprintf(outfile,"L36c = %6.2f\n", L[36]);
        L[37] = A[6];
        L[38] = A[7];
        L[39] = F[2];
        L[48] = 0.0;
       }
      else
       {
        fprintf(outfile," Sched-F, No: Take PropTax Credit\n\n");
        L[36] = 0.0;
	// fprintf(outfile,"L36c = %6.2f\n", L[36]);
        L[37] = Ab[6];
        L[38] = Ab[7];
        L[39] = Fb[2];
        L[48] = proptxcredit;
       }
    } /*Sched-F*/
  } /*Sched-A*/

 /* If no property tax was paid, be ensure prop.tax credit is set to 0 */
 if ( L36a == 0.0 ) L[48] = 0.0;

 fprintf(outfile,"L36c = %6.2f\n", L[36]);

 fprintf(outfile,"\n");  /* NJ Taxable Income.*/
 // L[37] = L[35] - L[36];  /* Handled above in Sched-1. */
 if (L[37] > 0.0)
  showline_wmsg(37,"NJ Taxable Income");

 // L[38] = TaxRateFunction( L[37], status );  /* Handled above in Sched-1. */
 showline_wmsg(38,"TAX");

 if (A[1] > 0.0)
  showline_wmsg(39,"( Credit for Taxes paid to other jurisdictions. )\n");

 L[40] = L[38] - L[39];
 showline_wmsg(40, "( Balance of Tax )");
 L[41] = L[40];
 showline(41);

 GetLineF( "L42", &L[42] );	/* Sheltered Workshop Tax Credit. */
 L[43] = L[41] - L[42];
 showline(43);

 GetLineF( "L44", &L[44] );	/* Use Tax Due on Out-of-State Purchases (pg 36). */
 GetLineF( "L45", &L[45] );	/* Penalty for underpayment of estimated tax. */

 L[46] = L[43] + L[44] + L[45];
 showline(46);			/* Total Tax + Penalty. */

 GetLine( "L47", &L[47] );
 showline_wmsg(47, "Total NJ Income Tax Withheld");

 showline_wmsg(48, "Property tax Credit");

 GetLineF( "L49", &L[49] );	/* NJ Estimated Tax Payments/Credit from 2005 return. */

 GetLineF( "L50", &L[50] );	/* NJ Earned Income Tax Credit. (See Sched pg 37.) */

 GetLineF( "L51", &L[51] );	/* EXCESS NJ UI/HC/WD Withheld, (See pg 38.) */

 GetLineF( "L52", &L[52] );	/* EXCESS NJ Disability Insurance Withheld, (See pg 38.) */

 L[53] = L[47] + L[48] + L[49] + L[50] + L[51] + L[52];
 showline_wmsg(53,"Total Payments/Credits");
 
 if (L[53] < L[46])
  {
   L[54] = L[46] - L[53];
   fprintf(outfile, "L54 = %6.2f	YOU OWE !!!\n", L[54] );
  }
 else
  {
   L[55] = L[53] - L[46];
   fprintf(outfile, "L55 = %6.2f	REFUND !!!\n", L[55] );

   L[63] = 0.0;
   showline_wmsg(63, "( Total Contributions from overpayment )");
   L[64] = L[55] - L[63];
   showline(64);
  }
 
 fclose(infile);
 fclose(outfile);
 Display_File( outfname );
 printf("\nResults written to file:  %s\n", outfname);
 return 0;
}
